@@ -2,29 +2,30 @@
 
 from __future__ import print_function, division, absolute_import
 
+import gzip
 import json
 import collections
-import multiprocessing
 try:  # py2
-    from urllib2 import urlopen, Request, URLError, HTTPError
+    from urllib2 import urlopen, Request
     from HTMLParser import HTMLParser
+    from urlparse import urljoin
 except ImportError:  # py3
     from urllib.request import urlopen, Request
-    from urllib.error import URLError, HTTPError
     from html.parser import HTMLParser
+    from urllib.parse import urljoin
 
-import concurrent.futures
 
 from .db import database
-from .unpack import top_level, unpack_html
+from .unpack import top_level, try_unpack_resp
 from .log import logger
 from .utils import Color, compare_version, cmp_to_key
+from .extractor import Extractor
 
 
-PYPI_URL = 'https://pypi.python.org/'
-PKG_URL = 'https://pypi.python.org/pypi/{0}'
-PKGS_URL = 'https://pypi.python.org/simple/'
-PKG_INFO_URL = 'https://pypi.python.org/pypi/{0}/json'
+PYPI_URL = 'https://pypi.org'
+PKG_URL = urljoin(PYPI_URL, '/pypi/{0}')
+PKGS_URL = urljoin(PYPI_URL, '/simple/')
+PKG_INFO_URL = urljoin(PYPI_URL, '/pypi/{0}/json')
 ACCEPTABLE_EXT = ('.whl', '.egg', '.tar.gz', '.tar.bz2', '.zip')
 
 
@@ -62,19 +63,21 @@ def update_db():
     """Update database."""
     print(Color.BLUE('Starting update database (this will take a while)...'))
     logger.info('Crawling "{0}" ...'.format(PKGS_URL))
-    data = download(PKGS_URL)
-    if not data:
-        logger.error('Operation abort ...')
+    try:
+        data = download(PKGS_URL)
+    except Exception:
+        logger.error("Fetch all packages got: ", exc_info=True)
+        print(Color.RED('Operation abort ...'))
         return
 
     logger.info('Extracting all packages ...')
-    pkg_names = _extract_html(unpack_html(data))
+    pkg_names = _extract_html(data)
     with database() as db:
         ignore_pkgs = db.query_package(None)
         pkg_names = list(set(pkg_names) - set(ignore_pkgs))
     extractor = Extractor(pkg_names)
-    extractor.extract(extract_pkg_info)
-    print(Color.BLUE('Update database done!'))
+    extractor.run(extract_pkg_info)
+    print(Color.BLUE('Operation done!'))
 
 
 def extract_pkg_info(pkg_name):
@@ -126,60 +129,14 @@ def _pkg_json_info(pkg_name):
     data = download(PKG_INFO_URL.format(pkg_name))
     if not data:  # 404
         return None
-    data = json.loads(data.decode('utf-8'))
+    data = json.loads(data)
     return data
 
 
-class Extractor(object):
-    """Extractor use thread pool execute tasks.
-
-    Can be used to extract /simple/<pkg_name> or /pypi/<pkg_name>/json.
-    """
-
-    def __init__(self, names, max_workers=None):
-        self._names = names
-        self._max_workers = max_workers or (multiprocessing.cpu_count() * 4)
-        self._futures = dict()
-
-    def extract(self, job):
-        """Extract url by package name."""
-        with concurrent.futures.ThreadPoolExecutor(
-                max_workers=self._max_workers) as executor:
-            for name in self._names:
-                self._futures[executor.submit(job, name)] = name
-
-            try:
-                self.wait_complete()
-            except KeyboardInterrupt:
-                print(Color.BLUE('** Canceling ...^... Please wait **'))
-                self.cancel()
-                executor.shutdown()
-                print(Color.BLUE('All tasks canceled!'))
-            else:
-                print(Color.BLUE('Extracting packages done!'))
-
-    def wait_complete(self):
-        """Wait for futures complete done."""
-        for future in concurrent.futures.as_completed(self._futures.keys()):
-            try:
-                error = future.exception()
-            except concurrent.futures.CancelledError:
-                break
-            name = self._futures[future]
-            if error is not None:
-                err_msg = 'Extracting "{0}", got: {1}'.format(name, error)
-                logger.error(err_msg)
-
-    def cancel(self):
-        print("+" * 60)
-        for future in self._futures:
-            future.cancel()
-
-
 # Fake headers, just in case.
 _HEADERS = {
     'Accept': '*/*',
-    'Accept-Encoding': 'gzip, deflate, sdch',
+    'Accept-Encoding': 'gzip',
     'Accept-Language': 'en-US,en;q=0.8,zh-CN;q=0.6,zh;q=0.4',
     'User-Agent': ('Mozilla/5.0 (X11; Linux x86_64; rv:13.0) '
                    'AppleWebKit/537.36 (KHTML, like Gecko) '
@@ -189,18 +146,11 @@ _HEADERS = {
 
 def download(url, headers=_HEADERS):
     """Download data from url."""
-    data = ''
+    resp = urlopen(Request(url, headers=headers))
     try:
-        f = urlopen(Request(url, headers=headers))
-    except HTTPError as e:
-        logger.error('Crawling "{0}", got: {1} {2}'.format(
-            url, e.code, e.reason))
-    except URLError as e:
-        logger.error('Crawling "{0}", got: {1}'.format(url, e.reason))
-    else:
-        data = f.read()
-        f.close()
-    return data
+        return try_unpack_resp(resp)
+    finally:
+        resp.close()
 
 
 def _extract_html(html):
